@@ -12,7 +12,10 @@ use crev_lib::id::LockedId;
 use crev_lib::{self, local::Local};
 use std::{
     collections::{HashMap, HashSet},
-    io::{self, BufRead, Write},
+    ffi::OsString,
+    fmt::Write as _,
+    io::{self, BufRead, Write as _},
+    panic,
     path::PathBuf,
 };
 use structopt::StructOpt;
@@ -33,26 +36,37 @@ mod review;
 mod shared;
 mod term;
 mod tokei;
+mod wot;
 
 use crate::{repo::*, review::*, shared::*};
 use crev_data::{proof, Id, TrustLevel};
 use crev_lib::TrustProofType;
 use crev_wot::{ProofDB, TrustSet, UrlOfId};
 
+/// Additional functions to extend `Local` by behaviors
+/// that are `cargo-crev` specific, like printing
+/// helpful diagnostic messages.
+trait LocalExt {
+    fn run_git_verbose(&self, args: Vec<OsString>) -> Result<std::process::ExitStatus>;
+}
+
+impl LocalExt for Local {
+    fn run_git_verbose(&self, args: Vec<OsString>) -> Result<std::process::ExitStatus> {
+        match self.run_git(args) {
+            Ok(o) => Ok(o),
+            Err(crev_lib::Error::GitUrlNotConfigured) => {
+                bail!("Id has no public URL set. Run `cargo crev id set-url <your-public-git-proof-repo>` and try again. See https://github.com/crev-dev/cargo-crev/discussions for help.");
+            }
+            Err(e) => Err(e)?,
+        }
+    }
+}
 pub fn repo_publish() -> Result<()> {
     let local = Local::auto_open()?;
-    let mut status = match local.run_git(vec!["diff".into(), "--exit-code".into()]) {
-        Err(err @ crev_lib::Error::GitUrlNotConfigured) => {
-            eprintln!("There is no public repo URL configured.");
-            print_crev_proof_repo_fork_help();
-            eprintln!("Run `cargo crev id set-url <your public repo>` and try again.");
-            return Err(err.into());
-        }
-        res => res?,
-    };
+    let mut status = local.run_git_verbose(vec!["diff".into(), "--exit-code".into()])?;
 
     if status.code().unwrap_or(-2) == 1 {
-        status = local.run_git(vec![
+        status = local.run_git_verbose(vec![
             "commit".into(),
             "-a".into(),
             "-m".into(),
@@ -61,17 +75,17 @@ pub fn repo_publish() -> Result<()> {
     }
 
     if status.code().unwrap_or(-1) == 0 {
-        status = local.run_git(vec!["pull".into(), "--rebase".into()])?;
+        status = local.run_git_verbose(vec!["pull".into(), "--rebase".into()])?;
     }
     if status.code().unwrap_or(-1) == 0 {
-        status = local.run_git(vec!["push".into()])?;
+        status = local.run_git_verbose(vec!["push".into()])?;
     }
     std::process::exit(status.code().unwrap_or(-159));
 }
 
 fn repo_update(args: opts::Update) -> Result<()> {
     let local = Local::auto_open()?;
-    let status = local.run_git(vec!["pull".into(), "--rebase".into()])?;
+    let status = local.run_git_verbose(vec!["pull".into(), "--rebase".into()])?;
     if !status.success() {
         std::process::exit(status.code().unwrap_or(-159));
     }
@@ -137,6 +151,7 @@ fn crate_review(args: opts::CrateReview) -> Result<()> {
             &args.common_proof_create,
             &args.diff,
             args.skip_activity_check || is_advisory || args.issue,
+            args.overrides,
             args.cargo_opts.clone(),
         )?;
         let has_public_url = local
@@ -195,18 +210,22 @@ fn print_ids<'a>(
     Ok(())
 }
 
+fn url_to_status_str<'a>(id_url: &UrlOfId<'a>) -> (&'static str, &'a str) {
+    match id_url {
+        UrlOfId::None => ("", ""),
+        UrlOfId::FromSelfVerified(url) => ("==", url.url.as_str()),
+        UrlOfId::FromSelf(url) => ("~=", url.url.as_str()),
+        UrlOfId::FromOthers(url) => ("??", url.url.as_str()),
+    }
+}
+
 fn print_mvp_ids<'a>(
     ids: impl Iterator<Item = (&'a Id, u64)>,
     trust_set: &TrustSet,
     db: &ProofDB,
 ) -> Result<()> {
     for (id, count) in ids {
-        let (status, url) = match db.lookup_url(id) {
-            UrlOfId::None => ("", ""),
-            UrlOfId::FromSelfVerified(url) => ("==", url.url.as_str()),
-            UrlOfId::FromSelf(url) => ("~=", url.url.as_str()),
-            UrlOfId::FromOthers(url) => ("??", url.url.as_str()),
-        };
+        let (status, url) = url_to_status_str(&db.lookup_url(id));
         println!(
             "{:>3} {} {:6} {} {}",
             count,
@@ -295,6 +314,7 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                     &args.common_proof_create,
                     args.level.unwrap_or(TrustLevel::Medium),
                     args.level.is_none(),
+                    args.overrides,
                 )?;
             }
             opts::Id::Untrust(args) => {
@@ -303,6 +323,7 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                     &args.common_proof_create,
                     TrustLevel::None,
                     true,
+                    args.overrides,
                 )?;
             }
             opts::Id::Distrust(args) => {
@@ -311,6 +332,7 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                     &args.common_proof_create,
                     TrustLevel::Distrust,
                     true,
+                    args.overrides,
                 )?;
             }
             opts::Id::Query(cmd) => match cmd {
@@ -351,7 +373,7 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                     let trust_set = db.calculate_trust_set(&for_id, &trust_params.into());
 
                     print_ids(
-                        trust_set.trusted_ids().filter(|id| {
+                        trust_set.iter_trusted_ids().filter(|id| {
                             trust_set.get_effective_trust_level(id)
                                 >= trust_level.trust_level.into()
                         }),
@@ -426,6 +448,7 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                 &args.common_proof_create,
                 args.level.unwrap_or(TrustLevel::Medium),
                 args.level.is_none(),
+                args.overrides,
             )?;
             // Make sure we have reviews for the new Ids we're trusting
             local.fetch_new_trusted(Default::default(), None)?;
@@ -435,31 +458,34 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                 let status = run_diff(&args)?;
                 std::process::exit(status.code().unwrap_or(-159));
             }
-            opts::Crate::Verify { crate_, opts } => {
-                return deps::verify_deps(crate_, opts);
+            opts::Crate::Verify(opts) => {
+                return deps::verify_deps(opts.crate_, opts.opts);
             }
-            opts::Crate::Mvp { crate_, opts } => {
-                deps::crate_mvps(crate_, opts)?;
+            opts::Crate::Mvp { crate_, opts, wot } => {
+                deps::crate_mvps(crate_, opts, wot)?;
             }
-            opts::Crate::Info { crate_, opts } => {
-                info::print_crate_info(crate_, opts)?;
+            opts::Crate::Info { crate_, opts, wot } => {
+                info::print_crate_info(crate_.auto_unrelated()?, opts, wot)?;
             }
             opts::Crate::Goto(args) => {
-                goto_crate_src(&args.crate_)?;
+                goto_crate_src(&args.crate_.auto_unrelated()?)?;
+            }
+            opts::Crate::Expand(args) => {
+                expand_crate_src(&args.crate_.auto_unrelated()?)?;
             }
             opts::Crate::Open(args) => {
-                handle_goto_mode_command(&args.common.clone(), |sel| {
-                    crate_open(sel, args.cmd, args.cmd_save)
+                handle_goto_mode_command(&args.common.clone(), |crate_| {
+                    crate_open(&crate_.clone().auto_unrelated()?, args.cmd, args.cmd_save)
                 })?;
             }
             opts::Crate::Clean(args) => {
                 if args.crate_.is_empty() && are_we_called_from_goto_shell().is_none() {
                     clean_all_crates_with_digest_mismatch()?;
                 } else {
-                    handle_goto_mode_command(&args, |sel| clean_crate(sel))?;
+                    handle_goto_mode_command(&args, clean_crate)?;
                 }
             }
-            opts::Crate::Dir(args) => show_dir(&args.common.crate_)?,
+            opts::Crate::Dir(args) => show_dir(&args.common.crate_.auto_unrelated()?)?,
 
             opts::Crate::Review(args) => crate_review(args)?,
             opts::Crate::Unreview(args) => {
@@ -492,6 +518,7 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
                         &args.common_proof_create,
                         &args.diff,
                         args.skip_activity_check || is_advisory || args.issue,
+                        args.overrides,
                         args.cargo_opts.clone(),
                     )
                 })?;
@@ -503,7 +530,15 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
         opts::Command::Config(args) => match args {
             opts::Config::Dir => {
                 let local = crev_lib::Local::auto_create_or_open()?;
-                println!("{}", local.get_root_path().display());
+                println!("{}", local.config_root().display());
+            }
+            opts::Config::DataDir => {
+                let local = crev_lib::Local::auto_create_or_open()?;
+                println!("{}", local.data_root().display());
+            }
+            opts::Config::CacheDir => {
+                let local = crev_lib::Local::auto_create_or_open()?;
+                println!("{}", local.cache_root().display());
             }
             opts::Config::Edit => {
                 let local = crev_lib::Local::auto_create_or_open()?;
@@ -545,8 +580,10 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
             }
             opts::Repo::Git(git) => {
                 let local = Local::auto_open()?;
-                let status = local.run_git(git.args)?;
-                std::process::exit(status.code().unwrap_or(-159));
+                let status = local.run_git_verbose(git.args)?;
+                return Ok(CommandExitStatus::CommandExitCode(
+                    status.code().unwrap_or(-159),
+                ));
             }
             opts::Repo::Query(args) => match args {
                 opts::RepoQuery::Review(args) => list_reviews(&args.crate_)?,
@@ -614,19 +651,24 @@ fn run_command(command: opts::Command) -> Result<CommandExitStatus> {
             }
         },
         opts::Command::Goto(args) => {
-            goto_crate_src(&args.crate_)?;
+            goto_crate_src(&args.crate_.auto_unrelated()?)?;
         }
         opts::Command::Open(args) => {
-            handle_goto_mode_command(&args.common.clone(), |sel| {
-                crate_open(sel, args.cmd, args.cmd_save)
+            handle_goto_mode_command(&args.common.clone(), |crate_| {
+                crate_open(&crate_.clone().auto_unrelated()?, args.cmd, args.cmd_save)
             })?;
         }
         opts::Command::Publish => repo_publish()?,
         opts::Command::Review(args) => crate_review(args)?,
         opts::Command::Update(args) => repo_update(args)?,
 
-        opts::Command::Verify { crate_, opts } => {
-            return deps::verify_deps(crate_, opts);
+        opts::Command::Wot(args) => match args {
+            opts::Wot::Log { wot } => {
+                crate::wot::print_log(wot)?;
+            }
+        },
+        opts::Command::Verify(opts) => {
+            return deps::verify_deps(opts.crate_, opts.opts);
         }
     }
 
@@ -750,11 +792,27 @@ fn set_trust_level_for_ids(
     common_proof_create: &crate::opts::CommonProofCreate,
     trust_level: TrustLevel,
     edit_interactively: bool,
+    show_override_suggestions: bool,
 ) -> Result<()> {
     let local = ensure_crev_id_exists_or_make_one()?;
     let unlocked_id = local.read_current_unlocked_id(&term::read_passphrase)?;
 
-    let mut trust = local.build_trust_proof(unlocked_id.as_public_id(), ids.to_vec(), trust_level)?;
+    let overrides = if ids.len() == 1 {
+        let db = local.load_db()?;
+
+        db.get_trust_proof_between(&unlocked_id.id.id, &ids[0])
+            .map(|trust_proof| trust_proof.override_.clone())
+            .unwrap_or(vec![])
+    } else {
+        vec![]
+    };
+
+    let mut trust = local.build_trust_proof(
+        unlocked_id.as_public_id(),
+        ids.to_vec(),
+        trust_level,
+        overrides,
+    )?;
 
     if edit_interactively {
         let extra_comment = if trust_level == TrustLevel::Distrust {
@@ -762,9 +820,28 @@ fn set_trust_level_for_ids(
         } else {
             None
         };
-        trust = edit::edit_proof_content_iteractively(&trust, None, None, extra_comment)?;
+        trust = edit::edit_proof_content_iteractively(&trust, None, None, extra_comment, |text| {
+            if show_override_suggestions && trust.override_.is_empty() {
+                writeln!(text, "# override:")?;
+            }
+
+            if show_override_suggestions {
+                let db = local.load_db()?;
+                for (id, trust_level) in ids.into_iter().flat_map(|id| db.get_reverse_trust_for(id))
+                {
+                    let (status, url) = url_to_status_str(&db.lookup_url(id));
+                    writeln!(text, "# - id-type: {}", "crev")?; // TODO: support other ids?
+                    writeln!(text, "#   id: {} # level: {}", id, trust_level)?;
+                    writeln!(text, "#   url: {} # {}", url, status)?;
+                    writeln!(text, "#   comment: \"\"")?;
+                }
+            }
+
+            Ok(())
+        })?;
     }
 
+    trust.touch_date();
     let proof = trust.sign_by(&unlocked_id)?;
 
     if common_proof_create.print_unsigned {
@@ -818,7 +895,7 @@ fn ids_from_string(id_strings: &[String]) -> Result<Vec<Id>> {
 fn load_stdin_with_prompt() -> Result<Vec<u8>> {
     let term = term::Term::new();
 
-    if term.stdin_is_tty {
+    if term.is_input_interactive() {
         eprintln!("Paste in the text and press Ctrl+D.")
     }
     let mut s = vec![];
@@ -911,12 +988,61 @@ fn main() {
         .init();
     let opts = opts::Opts::from_args();
     let opts::MainCommand::Crev(command) = opts.command;
-    match run_command(command) {
+    handle_command_result_and_panics(|| run_command(command))
+}
+
+fn is_possibly_broken_pipe_msg(s: &str) -> bool {
+    s.contains("Broken pipe") || s.contains("os error 32")
+}
+
+/**
+ * Handle command exit code and broken pipe IO errors.
+ *
+ * Set appropriate error code to the execution result.
+ *
+ * Broken pipe usually means that the user left `less` or some other pager
+ * and it's best to ignore such errors in both results and panics.
+ */
+// See https://github.com/crev-dev/cargo-crev/issues/287
+fn handle_command_result_and_panics(
+    f: impl FnOnce() -> Result<CommandExitStatus> + panic::UnwindSafe,
+) -> ! {
+    let hook = panic::take_hook();
+
+    // skip printing panic msg on broken pipe panics
+    panic::set_hook(Box::new(move |panic_info| {
+        if !is_possibly_broken_pipe_msg(&panic_info.to_string()) {
+            (hook)(panic_info);
+        }
+    }));
+
+    if let Err(panic_err) = panic::catch_unwind(|| match (f)() {
         Ok(CommandExitStatus::Success) => {}
         Ok(CommandExitStatus::VerificationFailed) => std::process::exit(-1),
+        Ok(CommandExitStatus::CommandExitCode(code)) => std::process::exit(code),
         Err(e) => {
+            if let Some(io_error) = e.root_cause().downcast_ref::<std::io::Error>() {
+                if io_error.kind() == std::io::ErrorKind::BrokenPipe {
+                    return;
+                }
+            }
             eprintln!("{:?}", e);
             std::process::exit(-2)
         }
+    }) {
+        let panic_str = if let Some(io_error) = panic_err.downcast_ref::<Box<&'static str>>() {
+            io_error.to_string()
+        } else if let Some(io_error) = panic_err.downcast_ref::<String>() {
+            io_error.to_string()
+        } else if let Some(io_error) = panic_err.downcast_ref::<&'static str>() {
+            io_error.to_string()
+        } else {
+            "".to_string()
+        };
+
+        if !is_possibly_broken_pipe_msg(&panic_str) {
+            panic::resume_unwind(panic_err);
+        }
     }
+    std::process::exit(0)
 }
