@@ -119,6 +119,7 @@ pub fn goto_crate_src(selector: &opts::CrateSelector) -> Result<()> {
     };
     let repo = Repo::auto_open_cwd_default()?;
     selector.ensure_name_given()?;
+
     let crate_id = repo.find_pkgid_by_crate_selector(selector)?;
     let crate_ = repo.get_crate(&crate_id)?;
     let crate_dir = crate_.root();
@@ -127,7 +128,7 @@ pub fn goto_crate_src(selector: &opts::CrateSelector) -> Result<()> {
     let local = crev_lib::Local::auto_create_or_open()?;
     local.record_review_activity(
         PROJECT_SOURCE_CRATES_IO,
-        &crate_.name().to_string(),
+        &crate_.name(),
         crate_version,
         &crev_lib::ReviewActivity::new_full(),
     )?;
@@ -143,12 +144,31 @@ pub fn goto_crate_src(selector: &opts::CrateSelector) -> Result<()> {
         .current_dir(crate_dir)
         .env("PWD", crate_dir)
         .env(GOTO_ORIGINAL_DIR_ENV, cwd)
-        .env(GOTO_CRATE_NAME_ENV, crate_name.to_string())
+        .env(GOTO_CRATE_NAME_ENV, crate_name)
         .env(GOTO_CRATE_VERSION_ENV, &crate_version.to_string());
 
     exec_into(command)
 }
 
+pub fn expand_crate_src(selector: &opts::CrateSelector) -> Result<()> {
+    use quote::ToTokens;
+
+    let repo = Repo::auto_open_cwd_default()?;
+    selector.ensure_name_given()?;
+    let crate_id = repo.find_pkgid_by_crate_selector(selector)?;
+    let crate_ = repo.get_crate(&crate_id)?;
+    let crate_dir = crate_.root();
+
+    // TODO: discover all the entrypoint, iterate over them, take care of formatting and
+    // comments
+    println!(
+        "{}",
+        syn_inline_mod::parse_and_inline_modules(&crate_dir.join("src/lib.rs")).into_token_stream()
+    );
+
+    Ok(())
+}
+
 pub fn ensure_known_owners_list_exists(local: &crev_lib::Local) -> Result<()> {
     let path = local.get_proofs_dir_path()?.join(KNOWN_CARGO_OWNERS_FILE);
     if !path.exists() {
@@ -187,7 +207,7 @@ pub fn clean_all_crates_with_digest_mismatch() -> Result<()> {
     let scanner = scan::Scanner::new(CrateSelector::default(), &opts::CrateVerify::default())?;
     let events = scanner.run(&RequiredDetails::none());
 
-    for stats in events.into_iter() {
+    for stats in events {
         if stats.details.accumulative_own.is_local_source_code {
             continue;
         }
@@ -654,6 +674,7 @@ pub enum CommandExitStatus {
     VerificationFailed,
     // Success, exit code 0
     Success,
+    CommandExitCode(i32),
 }
 
 pub fn get_crate_digest_mismatches(
@@ -707,7 +728,7 @@ pub fn lookup_crates(query: &str, count: usize) -> Result<()> {
     let local = crev_lib::Local::auto_create_or_open()?;
     let db = local.load_db()?;
 
-    let client = SyncClient::new("cargo-crev", std::time::Duration::from_millis(1000))?;
+    let client = SyncClient::new("cargo-crev", std::time::Duration::from_secs(1))?;
     let mut stats: Vec<_> = client
         .crates(ListOptions {
             sort: Sort::Downloads,
